@@ -177,6 +177,23 @@ class TestRunProcess(BasedirMixin, unittest.TestCase):
         self.assertTrue({'stdout': u"\N{SNOWMAN}"} in b.updates)
         self.assertTrue({'rc': 0} in b.updates, b.show())
 
+    @defer.inlineCallbacks
+    def testInvalidUTF8(self):
+        b = FakeWorkerForBuilder(self.basedir)
+        b.unicode_encoding = "utf-8"
+        s = runprocess.RunProcess(
+            b, stderrCommand("hello"), self.basedir, sendStderr=True)
+        pp = runprocess.RunProcessPP(s)
+        INVALID_UTF8 = b"\xff"
+        with self.assertRaises(UnicodeDecodeError):
+            INVALID_UTF8.decode('utf-8')
+        pp.outReceived(INVALID_UTF8)
+        yield s.start()
+        stdout = [up['stdout'] for up in b.updates if 'stdout' in up][0]
+        # On Python < 2.7 bytes is used, on Python >= 2.7 unicode
+        self.assertIn(stdout, (b'\xef\xbf\xbd', u'\ufffd'))
+        self.assertTrue({'rc': 0} in b.updates, b.show())
+
     @defer.inlineCallbacks
     def testKeepStderr(self):
         b = FakeWorkerForBuilder(self.basedir)
@@ -438,9 +455,9 @@ class TestRunProcess(BasedirMixin, unittest.TestCase):
 
     def testEnvironInt(self):
         b = FakeWorkerForBuilder(self.basedir)
-        self.assertRaises(RuntimeError, lambda:
-                          runprocess.RunProcess(b, stdoutCommand('hello'), self.basedir,
-                                                environ={"BUILD_NUMBER": 13}))
+        with self.assertRaises(RuntimeError):
+            runprocess.RunProcess(b, stdoutCommand('hello'), self.basedir,
+                                  environ={"BUILD_NUMBER": 13})
 
     def _test_spawnAsBatch(self, cmd, comspec):
 
@@ -656,6 +673,7 @@ class TestPOSIXKilling(BasedirMixin, unittest.TestCase):
         return self.do_test_pgroup(usePTY=False, useProcGroup=False,
                                    expectChildSurvival=True)
 
+    @defer.inlineCallbacks
     def do_test_pgroup(self, usePTY, useProcGroup=True,
                        expectChildSurvival=False):
         # test that a process group gets killed
@@ -687,16 +705,13 @@ class TestPOSIXKilling(BasedirMixin, unittest.TestCase):
         pidfiles_d.addCallback(kill)
 
         # check that both processes are dead after RunProcess is done
-        d = defer.gatherResults([pidfiles_d, runproc_d])
+        yield defer.gatherResults([pidfiles_d, runproc_d])
 
-        def check_dead(_):
-            self.assertDead(self.parent_pid)
-            if expectChildSurvival:
-                self.assertAlive(self.child_pid)
-            else:
-                self.assertDead(self.child_pid)
-        d.addCallback(check_dead)
-        return d
+        self.assertDead(self.parent_pid)
+        if expectChildSurvival:
+            self.assertAlive(self.child_pid)
+        else:
+            self.assertDead(self.child_pid)
 
     def test_double_fork_usePTY(self):
         return self.do_test_double_fork(usePTY=True)
@@ -710,6 +725,7 @@ class TestPOSIXKilling(BasedirMixin, unittest.TestCase):
         return self.do_test_double_fork(usePTY=False, useProcGroup=False,
                                         expectChildSurvival=True)
 
+    @defer.inlineCallbacks
     def do_test_double_fork(self, usePTY, useProcGroup=True,
                             expectChildSurvival=False):
         # when a spawned process spawns another process, and then dies itself
@@ -743,16 +759,13 @@ class TestPOSIXKilling(BasedirMixin, unittest.TestCase):
         pidfiles_d.addCallback(kill)
 
         # check that both processes are dead after RunProcess is done
-        d = defer.gatherResults([pidfiles_d, runproc_d])
+        yield defer.gatherResults([pidfiles_d, runproc_d])
 
-        def check_dead(_):
-            self.assertDead(self.parent_pid)
-            if expectChildSurvival:
-                self.assertAlive(self.child_pid)
-            else:
-                self.assertDead(self.child_pid)
-        d.addCallback(check_dead)
-        return d
+        self.assertDead(self.parent_pid)
+        if expectChildSurvival:
+            self.assertAlive(self.child_pid)
+        else:
+            self.assertDead(self.child_pid)
 
 
 class TestLogging(BasedirMixin, unittest.TestCase):
@@ -830,17 +843,46 @@ class TestLogFileWatcher(BasedirMixin, unittest.TestCase):
 
     def test_statFile_missing(self):
         rp = self.makeRP()
-        if os.path.exists('statfile.log'):
-            os.remove('statfile.log')
-        lf = runprocess.LogFileWatcher(rp, 'test', 'statfile.log', False)
-        self.assertFalse(lf.statFile(), "statfile.log doesn't exist")
+        test_filename = 'test_runprocess_test_statFile_missing.log'
+        if os.path.exists(test_filename):
+            os.remove(test_filename)
+        lf = runprocess.LogFileWatcher(rp, 'test', test_filename, False)
+        self.assertFalse(lf.statFile(), "{} doesn't exist".format(test_filename))
 
     def test_statFile_exists(self):
         rp = self.makeRP()
-        with open('statfile.log', 'w') as f:
-            f.write('hi')
-        lf = runprocess.LogFileWatcher(rp, 'test', 'statfile.log', False)
-        st = lf.statFile()
-        self.assertEqual(
-            st and st[2], 2, "statfile.log exists and size is correct")
-        os.remove('statfile.log')
+        test_filename = 'test_runprocess_test_statFile_exists.log'
+        try:
+            with open(test_filename, 'w') as f:
+                f.write('hi')
+            lf = runprocess.LogFileWatcher(rp, 'test', test_filename, False)
+            st = lf.statFile()
+            self.assertEqual(
+                st and st[2], 2, "statfile.log exists and size is correct")
+        finally:
+            os.remove(test_filename)
+
+    def test_invalid_utf8(self):
+        # create the log file watcher first
+        rp = self.makeRP()
+        test_filename = 'test_runprocess_test_invalid_utf8.log'
+
+        try:
+            lf = runprocess.LogFileWatcher(rp, 'test', test_filename,
+                                           follow=False, poll=False)
+            # now write to the log file
+            INVALID_UTF8 = b'before\xffafter'
+            with open(test_filename, 'wb') as f:
+                f.write(INVALID_UTF8)
+            # the watcher picks up the changed log
+            lf.poll()
+            # flush she buffer
+            rp._sendBuffers()
+            # the log file content was captured and the invalid byte replaced with \ufffd (the
+            # replacement character, often a black diamond with a white question mark)
+            REPLACED = u'before\ufffdafter'
+            self.assertEqual(rp.builder.updates, [{'log': ('test', REPLACED)}])
+
+        finally:
+            lf.stop()
+            os.remove(f.name)
